/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.pdfconv;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import org.jpedal.PdfDecoder;

import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;

public class ImageWriter
{
    private double _maxHeigth=-1;
    private double _maxWidth=-1;
    private double _scaleFactor=1;
    private float _quality=0.85f;

    public void setQuality(String quality)
    {
        try
        {
            _quality=Float.parseFloat(quality);
        }
        catch (NumberFormatException e)
        {
        }
    }

    public void setScaleFactor(String scaleFactor)
    {
        try
        {
            _scaleFactor=Double.parseDouble(scaleFactor);
        }
        catch (NumberFormatException e)
        {
        }
    }

    public void setMaxHeigth(String maxHeigth)
    {
        try
        {
            _maxHeigth=Integer.parseInt(maxHeigth);
        }
        catch (NumberFormatException e)
        {
        }
    }

    public void setMaxWidht(String maxWidth)
    {
        try
        {
            _maxWidth=Integer.parseInt(maxWidth);
        }
        catch (NumberFormatException e)
        {
        }
    }

    public void writeImage(BufferedImage targetImage, PdfDecoder pdfDecoder, String targetDir, String targetName)
    {
//        System.out.println("image: width="+targetImage.getWidth()+", heigth="+targetImage.getHeight());
        double heigth=targetImage.getHeight();
        double width=targetImage.getWidth();
        boolean flipImage=false;
        AffineTransform at=new AffineTransform();

        String version=pdfDecoder.version;
        double d=Double.parseDouble(version);
        if (d>1.81) // flipping the image is needed since jpedal version 1.81
        {
            System.out.println("flipping image");
            flipImage=true;
            at.scale(1, -1); // flip image upside-down
            at.translate(0, -targetImage.getHeight());
        }
        boolean scaleImage=false;
        double scaleFactor=_scaleFactor;
        if (-1!=_maxHeigth||-1!=_maxWidth||1!=_scaleFactor)
        {
//            System.out.println("checking for scale");
            if (-1!=_maxWidth&&_maxWidth<width)
            {
                d=_maxWidth/width;
                if (d<scaleFactor)
                    scaleFactor=d;
                scaleImage=true;
//                System.out.println("scale width="+scaleFactor);
            }
            if (-1!=_maxHeigth&&_maxHeigth<heigth)
            {
                d=_maxHeigth/heigth;
//                System.out.println("scale heigth="+d);
                if (d<scaleFactor)
                    scaleFactor=d;
                scaleImage=true;
            }
            if (1!=_scaleFactor)
                scaleImage=true;
//            System.out.println("scale factor="+scaleFactor);
            at.scale(scaleFactor, scaleFactor);
        }
        if (flipImage||scaleImage)
        {
//            System.out.println("doing transformation");
            AffineTransformOp ato=new AffineTransformOp(at, null);
            targetImage=ato.filter(targetImage, null);
        }
//        System.out.println("image: width="+targetImage.getWidth()+", heigth="+targetImage.getHeight());
        saveStoredJPEGImage(targetDir+targetName, targetImage);
    }

    private boolean saveStoredJPEGImage(String current_image, BufferedImage image)
    {
        String file_name=current_image;
        try
        {
            BufferedOutputStream out=new BufferedOutputStream(new FileOutputStream(new File(file_name)));
            JPEGEncodeParam param=JPEGCodec.getDefaultJPEGEncodeParam(image);
            param.setQuality(_quality, true);
            JPEGImageEncoder as_jpeg=JPEGCodec.createJPEGEncoder(out, param);
            as_jpeg.encode(image);
            out.close();
        }
        catch (Exception e)
        {
        }
        return true;
    }
}
